import path from 'node:path';
import yargsParser from 'yargs-parser';
import { BaseProject } from '@thing.js/cli-utils';
import webpack from 'webpack';
import { createRequire } from 'node:module';
import { fileURLToPath } from 'node:url';
import { CleanWebpackPlugin } from 'clean-webpack-plugin';
import CopyPlugin from 'copy-webpack-plugin';
import { merge } from 'webpack-merge';

const __dirname = path.dirname(fileURLToPath(import.meta.url));
function buildConfig(pkgs = [], userWebpackConfig = {}, compiler) {
  var _a, _b;
  const entry = {};
  const jsLoaderIncludes = [];
  Object.values(pkgs).forEach((pkg) => {
    entry[pkg.dirName + "/index"] = pkg.inputPath;
    jsLoaderIncludes.push(pkg.inputPath);
  });
  const plugins = [new CleanWebpackPlugin()];
  if (pkgs == null ? void 0 : pkgs.length) {
    const copyPlugin = new CopyPlugin({
      patterns: Object.values(pkgs).map((pkg) => {
        return {
          from: pkg.dirPath,
          to: pkg.outputDirPath,
          globOptions: {
            dot: true,
            gitignore: false,
            ignore: ["**/components/**", "**/index.js"]
          },
          transform(content, absoluteFrom) {
            var _a2;
            if (absoluteFrom === pkg.pkgJsonPath && ((_a2 = pkg.custom) == null ? void 0 : _a2.noName)) {
              const json = JSON.parse(content.toString());
              json.name = "plugin";
              return JSON.stringify(json, null, 2);
            }
            return content;
          }
        };
      })
    });
    plugins.push(copyPlugin);
  }
  const outputPath = pkgs[0].rootOutputDirPath;
  return merge(
    {
      entry,
      output: {
        path: outputPath,
        library: {
          name: "[name]",
          type: "umd"
        }
      },
      mode: "production",
      watch: (_b = (_a = compiler.command) == null ? void 0 : _a.opts) == null ? void 0 : _b.watch,
      stats: "errors-only",
      plugins,
      module: {
        rules: [
          {
            test: /\.(png|jpe?g|gif)$/i,
            use: [
              {
                loader: createRequire(import.meta.url).resolve("file-loader"),
                options: {
                  name(resourcePath) {
                    const relativePath = path.relative(outputPath, resourcePath);
                    const paths = relativePath.split(path.sep);
                    const pIndex = paths.findIndex((p) => p === "packages");
                    const filename = paths.slice(pIndex + 1);
                    return filename.join(path.sep);
                  }
                }
              }
            ]
          },
          {
            test: /\.js$/,
            include: jsLoaderIncludes,
            use: [
              {
                loader: path.resolve(__dirname, "./loader/add-props-loader")
              },
              {
                loader: path.resolve(
                  __dirname,
                  "./loader/export-classes-loader"
                )
              }
            ]
          }
        ]
      }
    },
    userWebpackConfig
  );
}

class PluginProject extends BaseProject {
  getPackageInfo(dirPath) {
    const pkgJsonPath = path.join(dirPath, "bundle.json");
    let inputPath;
    const pkgJson = this.resolveJsonFile(pkgJsonPath);
    if (pkgJson.main) {
      inputPath = path.resolve(dirPath, pkgJson.main);
    } else {
      throw new Error(`${pkgJsonPath} \u4E2D\u6CA1\u6709\u627E\u5230 main \u5B57\u6BB5`);
    }
    const noName = !pkgJson.name;
    if (noName) {
      pkgJson.name = "plugin";
    }
    return {
      inputPath,
      pkgJsonPath,
      pkgJson,
      custom: {
        noName
      }
    };
  }
  onBuild() {
    const { userPkgList, thingConfig } = this.compiler;
    return new Promise((resolve) => {
      const config = buildConfig(
        userPkgList,
        thingConfig.configureWebpack,
        this.compiler
      );
      webpack(config, async (err, stats) => {
        if (err) {
          throw new Error(err.message);
        }
        if (stats) {
          process.stdout.write(
            stats.toString({
              colors: true,
              modules: false,
              children: false,
              chunks: false,
              chunkModules: false
            })
          );
        }
        resolve();
      });
    });
  }
}
const index = (api) => {
  api.registerCommand(
    "build",
    {
      description: "\u6253\u5305\u8D44\u6E90",
      options: {
        "-p, --packages": "\u6307\u5B9A\u8981\u6253\u5305\u7684\u8D44\u6E90\u540D\u79F0, \u591A\u4E2A\u4F7F\u7528\u7A7A\u683C\u5206\u5F00",
        "-w, --watch": "\u76D1\u542C\u8D44\u6E90\u53D8\u5316"
      },
      usage: `thing-cli-service build [option]`
    },
    (args, rawArgs) => {
      const command = {
        name: "build",
        opts: yargsParser(rawArgs, {
          array: ["packages"],
          alias: {
            packages: "p",
            watch: "w"
          }
        })
      };
      new PluginProject(command);
    }
  );
};

export { index as default };
